@@ -534,16 +534,35 @@ class Server(TmuxRelationalObject["Session", "SessionDict"], EnvironmentMixin):
         Raises
         ------
         :exc:`exc.BadSessionName`
-        """
-        session_check_name(session_name)
-        assert session_name is not None
 
-        if self.has_session(session_name):
-            if kill_session:
-                self.cmd("kill-session", "-t%s" % session_name)
-                logger.info("session %s exists. killed it." % session_name)
-            else:
-                raise exc.TmuxSessionExists("Session named %s exists" % session_name)
+        Examples
+        --------
+        Sessions can be created without a session name (0.14.2+):
+
+        >>> server.new_session()
+        Session($2 2)
+
+        Creating them in succession will enumerate IDs (via tmux):
+
+        >>> server.new_session()
+        Session($3 3)
+
+        With a `session_name`:
+
+        >>> server.new_session(session_name='my session')
+        Session($4 my session)
+        """
+        if session_name is not None:
+            session_check_name(session_name)
+
+            if self.has_session(session_name):
+                if kill_session:
+                    self.cmd("kill-session", "-t%s" % session_name)
+                    logger.info("session %s exists. killed it." % session_name)
+                else:
+                    raise exc.TmuxSessionExists(
+                        "Session named %s exists" % session_name
+                    )
 
         logger.debug("creating session %s" % session_name)
 
@@ -556,11 +575,13 @@ class Server(TmuxRelationalObject["Session", "SessionDict"], EnvironmentMixin):
             del os.environ["TMUX"]
 
         tmux_args: t.Tuple[t.Union[str, int], ...] = (
-            "-s%s" % session_name,
             "-P",
             "-F%s" % formats.FORMAT_SEPARATOR.join(tmux_formats),  # output
         )
 
+        if session_name is not None:
+            tmux_args += (f"-s{session_name}",)
+
         if not attach:
             tmux_args += ("-d",)
 
