@@ -2,48 +2,99 @@
 import contextlib
 import logging
 import os
-import tempfile
+import random
 import time
+import types
+import typing as t
+from typing import Callable, Optional
+
+from libtmux.server import Server
+
+from .exc import WaitTimeout
 
 logger = logging.getLogger(__name__)
 
+if t.TYPE_CHECKING:
+    from libtmux.session import Session
+    from libtmux.window import Window
+
 TEST_SESSION_PREFIX = "libtmux_"
 RETRY_TIMEOUT_SECONDS = int(os.getenv("RETRY_TIMEOUT_SECONDS", 8))
+RETRY_INTERVAL_SECONDS = float(os.getenv("RETRY_INTERVAL_SECONDS", 0.05))
 
-namer = tempfile._RandomNameSequence()
+
+class RandomStrSequence:
+    def __init__(
+        self, characters: str = "abcdefghijklmnopqrstuvwxyz0123456789_"
+    ) -> None:
+        self.characters: str = characters
+
+    def __iter__(self) -> "RandomStrSequence":
+        return self
+
+    def __next__(self) -> str:
+        return "".join(random.sample(self.characters, k=8))
+
+
+namer = RandomStrSequence()
 current_dir = os.path.abspath(os.path.dirname(__file__))
 example_dir = os.path.abspath(os.path.join(current_dir, "..", "examples"))
 fixtures_dir = os.path.realpath(os.path.join(current_dir, "fixtures"))
 
 
-def retry(seconds=RETRY_TIMEOUT_SECONDS):
+def retry_until(
+    fun: Callable[[], bool],
+    seconds: float = RETRY_TIMEOUT_SECONDS,
+    *,
+    interval: float = RETRY_INTERVAL_SECONDS,
+    raises: Optional[bool] = True,
+) -> bool:
     """
-    Retry a block of code until a time limit or ``break``.
+    Retry a function until a condition meets or the specified time passes.
 
     Parameters
     ----------
-    seconds : int
-        Seconds to retry, defaults to ``RETRY_TIMEOUT_SECONDS``, which is
-        configurable via environmental variables.
-
-    Returns
-    -------
-    bool
-        True if time passed since retry() invoked less than seconds param.
+    fun : callable
+        A function that will be called repeatedly until it returns ``True``  or
+        the specified time passes.
+    seconds : float
+        Seconds to retry. Defaults to ``8``, which is configurable via
+        ``RETRY_TIMEOUT_SECONDS`` environment variables.
+    interval : float
+        Time in seconds to wait between calls. Defaults to ``0.05`` and is
+        configurable via ``RETRY_INTERVAL_SECONDS`` environment variable.
+    raises : bool
+        Wether or not to raise an exception on timeout. Defaults to ``True``.
 
     Examples
     --------
 
-    >>> while retry():
-    ...     p = w.attached_pane
+    >>> def fn():
+    ...     p = session.attached_window.attached_pane
     ...     p.server._update_panes()
-    ...     if p.current_path == pane_path:
-    ...         break
+    ...     return p.current_path is not None
+
+    >>> retry_until(fn)
+    True
+
+    In pytest:
+
+    >>> assert retry_until(fn, raises=False)
     """
-    return (lambda: time.time() < time.time() + seconds)()
+    ini = time.time()
+
+    while not fun():
+        end = time.time()
+        if end - ini >= seconds:
+            if raises:
+                raise WaitTimeout()
+            else:
+                return False
+        time.sleep(interval)
+    return True
 
 
-def get_test_session_name(server, prefix=TEST_SESSION_PREFIX):
+def get_test_session_name(server: "Server", prefix: str = TEST_SESSION_PREFIX) -> str:
     """
     Faker to create a session name that doesn't exist.
 
@@ -67,7 +118,9 @@ def get_test_session_name(server, prefix=TEST_SESSION_PREFIX):
     return session_name
 
 
-def get_test_window_name(session, prefix=TEST_SESSION_PREFIX):
+def get_test_window_name(
+    session: "Session", prefix: t.Optional[str] = TEST_SESSION_PREFIX
+) -> str:
     """
     Faker to create a window name that doesn't exist.
 
@@ -86,15 +139,18 @@ def get_test_window_name(session, prefix=TEST_SESSION_PREFIX):
     str
         Random window name guaranteed to not collide with current ones.
     """
+    assert prefix is not None
     while True:
         window_name = prefix + next(namer)
-        if not session.find_where(window_name=window_name):
+        if not session.find_where({"window_name": window_name}):
             break
     return window_name
 
 
 @contextlib.contextmanager
-def temp_session(server, *args, **kwargs):
+def temp_session(
+    server: Server, *args: t.Any, **kwargs: t.Any
+) -> t.Generator["Session", t.Any, t.Any]:
     """
     Return a context manager with a temporary session.
 
@@ -125,6 +181,7 @@ def temp_session(server, *args, **kwargs):
 
     >>> with temp_session(server) as session:
     ...     session.new_window(window_name='my window')
+    Window(@... ...:..., Session($... ...))
     """
 
     if "session_name" in kwargs:
@@ -143,7 +200,9 @@ def temp_session(server, *args, **kwargs):
 
 
 @contextlib.contextmanager
-def temp_window(session, *args, **kwargs):
+def temp_window(
+    session: "Session", *args: t.Any, **kwargs: t.Any
+) -> t.Generator["Window", t.Any, t.Any]:
     """
     Return a context manager with a temporary window.
 
@@ -173,7 +232,13 @@ def temp_window(session, *args, **kwargs):
     --------
 
     >>> with temp_window(session) as window:
-    ...     my_pane = window.split_window()
+    ...     window
+    Window(@... ...:..., Session($... ...))
+
+
+    >>> with temp_window(session) as window:
+    ...     window.split_window()
+    Pane(%... Window(@... ...:..., Session($... ...)))
     """
 
     if "window_name" not in kwargs:
@@ -185,11 +250,13 @@ def temp_window(session, *args, **kwargs):
 
     # Get ``window_id`` before returning it, it may be killed within context.
     window_id = window.get("window_id")
+    assert window_id is not None
+    assert isinstance(window_id, str)
 
     try:
-        yield session
+        yield window
     finally:
-        if session.findWhere(window_id=window_id):
+        if session.find_where({"window_id": window_id}):
             window.kill_window()
     return
 
@@ -215,27 +282,32 @@ class EnvironmentVarGuard:
        Created October 12th, 2015. Accessed April 7th, 2018.
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
         self._environ = os.environ
-        self._unset = set()
-        self._reset = dict()
+        self._unset: t.Set[str] = set()
+        self._reset: t.Dict[str, str] = dict()
 
-    def set(self, envvar, value):
+    def set(self, envvar: str, value: str) -> None:
         if envvar not in self._environ:
             self._unset.add(envvar)
         else:
             self._reset[envvar] = self._environ[envvar]
         self._environ[envvar] = value
 
-    def unset(self, envvar):
+    def unset(self, envvar: str) -> None:
         if envvar in self._environ:
             self._reset[envvar] = self._environ[envvar]
             del self._environ[envvar]
 
-    def __enter__(self):
+    def __enter__(self) -> "EnvironmentVarGuard":
         return self
 
-    def __exit__(self, *ignore_exc):
+    def __exit__(
+        self,
+        exc_type: t.Union[t.Type[BaseException], None],
+        exc_value: t.Optional[BaseException],
+        exc_tb: t.Optional[types.TracebackType],
+    ) -> None:
         for envvar, value in self._reset.items():
             self._environ[envvar] = value
         for unset in self._unset:
